package com.sams.jws.chapter15;
/*
 * FileService
 * Process JAXM requests for files named in a SOAP message.If the file is 
 * a directory, then list its contents--including characteristics of files in 
 * the directory--in the SOAP body. If the file is a regular file, return it 
 * as an attachment and include its characteristics in the SOAP body.
 * Typical requests look like the following:
 * <Envelope xmlns="http://schemas.xmlsoap.org/soap/envelope/">
 * <Header><CmdName mustUnderstand="1">FileService</CmdName></Header>
 * <Body><FileName>C:\temp\dir\a.txt</FileName></Body>
 * </Envelope>
 */
// General JDK
import java.io.IOException;
import java.io.File;
import java.util.Iterator;
import java.util.Hashtable;
// JAXM
import javax.xml.soap.MessageFactory;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPElement;
import javax.xml.soap.SOAPEnvelope;
import javax.xml.soap.SOAPHeader;
import javax.xml.soap.SOAPHeaderElement;
import javax.xml.soap.SOAPBody;
import javax.xml.soap.SOAPFault;
import javax.xml.soap.SOAPException;
import javax.xml.soap.AttachmentPart;
import javax.xml.messaging.JAXMServlet;
import javax.xml.messaging.ReqRespListener;
// JAF
import javax.activation.DataHandler;
import javax.activation.FileDataSource;
// Servlet
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;

public class FileService extends JAXMServlet implements ReqRespListener {
    static final boolean DEBUG = true;
    MessageFactory messageFactory = null;
    private ServletContext servletContext = null;
    private static Hashtable errorCodes = new Hashtable();
    static {
        // Initialize all error codes and messages here.
        errorCodes.put("100", "Incorrect command name sent to FileService.");
        errorCodes.put("200", "Missing FileName tag or illegal content.");
        errorCodes.put("300", "File does not exist.");
        errorCodes.put("400", "Cannot read file.");
    }

    public void init(ServletConfig servletConfig) throws ServletException {
        super.init(servletConfig);
        servletContext = servletConfig.getServletContext();
        try {
            messageFactory = MessageFactory.newInstance();
        } catch (SOAPException e) {
            System.err.println(
                "FileService caught exception instantiating MessageFactory:");
            e.printStackTrace();
        }
    }	// init()
    /*
     * renderFault
     * A fault has occured. Return fault details as response.
     */
    public void renderFault(
        SOAPHeaderElement responseType,
        SOAPBody responseSoapBody,
        String code,
        String param)
        throws SOAPException {

        System.err.println("ERROR Code: " + code);
        System.err.println("ERROR Desc: " + errorCodes.get(code) + " " + param);

        responseType.addTextNode("fault");
        SOAPFault soapFault = responseSoapBody.addFault();
        soapFault.setFaultCode(code);
        soapFault.setFaultString((String) errorCodes.get(code) + " " + param);
    }	// renderFault()
    // Version of renderFault() method without additional param (eg: file name).
    public void renderFault(
        SOAPHeaderElement responseType,
        SOAPBody responseSoapBody,
        String code)
        throws SOAPException {
        renderFault(responseType, responseSoapBody, code, "");
    }	// renderFault()

    /*
     * onMessage
     * Process JAX request to retrieve file. 
     */
    public SOAPMessage onMessage(SOAPMessage requestSoapMsg) {

        try {
            if (DEBUG) {
                System.out.println(
                    "FileService received the following message in onMessage():");
                requestSoapMsg.writeTo(System.out);
                System.out.println("\n");
            }

            // Create response
            SOAPMessage responseSoapMsg = messageFactory.createMessage();
            SOAPEnvelope responseSoapEnv = responseSoapMsg.getSOAPPart().getEnvelope();
            SOAPHeader responseSoapHdr = responseSoapEnv.getHeader();
            SOAPBody responseSoapBody = responseSoapEnv.getBody();

            SOAPHeaderElement responseType =
                responseSoapHdr.addHeaderElement(responseSoapEnv.createName("ResponseType"));

            // Extract contents of query, validating in process
            SOAPEnvelope requestSoapEnv = requestSoapMsg.getSOAPPart().getEnvelope();
            SOAPHeader requestSoapHdr = requestSoapEnv.getHeader();
            SOAPBody requestSoapBody = requestSoapEnv.getBody();

            // Validate the CmdName header
            String cmd = DocumentUtils.findValueOfChild(requestSoapHdr, "CmdName");
            if ((cmd == null) || (!cmd.equals("FileService"))) {
                // Could not locate CmdName tag
                renderFault(responseType, responseSoapBody, "100");
                return responseSoapMsg;
            } 

            // Get filename parameter
            String fileName = DocumentUtils.findValueOfChild(requestSoapBody, "FileName");
            if ((fileName == null) || (fileName.indexOf("..") > -1)) {
                // No parameter, or attempt to navigate out of webroot
                renderFault(responseType, responseSoapBody, "200");
                return responseSoapMsg;
            } 

            // Translate to absolute path with respect to web root.
            fileName = servletContext.getRealPath(fileName);
            System.out.println("Opening local fileName: " + fileName);
            File file = new File(fileName);
            if (!file.exists()) {
                // File doesn't exist.
                renderFault(responseType, responseSoapBody, "300", file.getName());
                return responseSoapMsg;
            } 

            if (!file.canRead()) {
                // File is inaccessible.
                renderFault(responseType, responseSoapBody, "400", file.getName());
                return responseSoapMsg;
            } 

            if (file.isDirectory()) {
                // List directory contents in SOAP body
                responseType.addTextNode("directoryListing");
                SOAPElement listElement = responseSoapBody.addChildElement("List");

                File[] fileList = file.listFiles();
                for (int j = 0; j < fileList.length; j++) {
                    FileElement fileElement = new FileElement(fileList[j]);
                    fileElement.marshalFile(listElement);
                } 
                return responseSoapMsg;

            } 

            if (file.isFile()) {
                // Return file contents as attachment and file characteristics
                // in SOAP body
                responseType.addTextNode("attachedFile");
                FileElement fileElement = new FileElement(file);
                fileElement.marshalFile(responseSoapBody);

                // Add an attachment to the message
                DataHandler dataHandler = new DataHandler(new FileDataSource(file));
                AttachmentPart attachmentPart =
                    responseSoapMsg.createAttachmentPart(dataHandler);
                attachmentPart.setContentType(dataHandler.getContentType());
                // Add extra MIME header with file name
                attachmentPart.addMimeHeader("FileName", file.getName());
                responseSoapMsg.addAttachmentPart(attachmentPart);
                return responseSoapMsg;

            } 

            // General exceptions potentially thrown when working with a SOAPMessages. 
        } catch (SOAPException e) {
            System.err.println(
                "FileService encountered error in processing or replying to msg");
            e.printStackTrace();
            return null;
        } catch (IOException e) {
            System.err.println(
                "FileService IOException when trying to write SOAPMessage: ");
            e.printStackTrace();
            return null;
        } 
        return null;
    } // onMessage
} // Class FileService